# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: types.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='types.proto',
  package='',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x0btypes.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\"\n\x0bStringInput\x12\x13\n\x0bStringValue\x18\x01 \x01(\t\" \n\tMinerList\x12\x13\n\x0bpublic_keys\x18\x01 \x03(\x0c\"\x15\n\x04Hash\x12\r\n\x05Value\x18\x01 \x01(\x0c\"\x18\n\x07\x41\x64\x64ress\x12\r\n\x05Value\x18\x01 \x01(\x0c\"\xa3\x01\n\x0bTransaction\x12\x16\n\x04\x46rom\x18\x01 \x01(\x0b\x32\x08.Address\x12\x14\n\x02To\x18\x02 \x01(\x0b\x32\x08.Address\x12\x16\n\x0eRefBlockNumber\x18\x03 \x01(\x03\x12\x16\n\x0eRefBlockPrefix\x18\x04 \x01(\x0c\x12\x12\n\nMethodName\x18\x05 \x01(\t\x12\x0e\n\x06Params\x18\x06 \x01(\x0c\x12\x12\n\tSignature\x18\x90N \x01(\x0c\"\xca\x02\n\x14\x45lectionVotingRecord\x12\x17\n\x05voter\x18\x01 \x01(\x0b\x32\x08.Address\x12\x11\n\tcandidate\x18\x02 \x01(\t\x12\x0e\n\x06\x61mount\x18\x03 \x01(\x12\x12\x13\n\x0bterm_number\x18\x04 \x01(\x12\x12\x16\n\x07vote_id\x18\x05 \x01(\x0b\x32\x05.Hash\x12\x11\n\tlock_time\x18\x07 \x01(\x12\x12\x34\n\x10unlock_timestamp\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x36\n\x12withdraw_timestamp\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0evote_timestamp\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0cis_withdrawn\x18\r \x01(\x08\"\xa7\x02\n\x0b\x45lectorVote\x12\'\n\x18\x61\x63tive_voting_record_ids\x18\x01 \x03(\x0b\x32\x05.Hash\x12*\n\x1bwithdrawn_voting_record_ids\x18\x02 \x03(\x0b\x32\x05.Hash\x12!\n\x19\x61\x63tive_voted_votes_amount\x18\x03 \x01(\x12\x12\x1e\n\x16\x61ll_voted_votes_amount\x18\x04 \x01(\x12\x12\x34\n\x15\x61\x63tive_voting_records\x18\x05 \x03(\x0b\x32\x15.ElectionVotingRecord\x12\x36\n\x17withdrawn_votes_records\x18\x06 \x03(\x0b\x32\x15.ElectionVotingRecord\x12\x12\n\npublic_key\x18\x07 \x01(\x0c\"\xdf\x02\n\rCandidateVote\x12\x30\n!obtained_active_voting_record_ids\x18\x01 \x03(\x0b\x32\x05.Hash\x12\x33\n$obtained_withdrawn_voting_record_ids\x18\x02 \x03(\x0b\x32\x05.Hash\x12*\n\"obtained_active_voted_votes_amount\x18\x03 \x01(\x12\x12\'\n\x1f\x61ll_obtained_voted_votes_amount\x18\x04 \x01(\x12\x12=\n\x1eobtained_active_voting_records\x18\x05 \x03(\x0b\x32\x15.ElectionVotingRecord\x12?\n obtained_withdrawn_votes_records\x18\x06 \x03(\x0b\x32\x15.ElectionVotingRecord\x12\x12\n\npublic_key\x18\x07 \x01(\x0c\"\x1f\n\x0ePublicKeysList\x12\r\n\x05value\x18\x01 \x03(\x0c\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])




_STRINGINPUT = _descriptor.Descriptor(
  name='StringInput',
  full_name='StringInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='StringValue', full_name='StringInput.StringValue', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=48,
  serialized_end=82,
)


_MINERLIST = _descriptor.Descriptor(
  name='MinerList',
  full_name='MinerList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='public_keys', full_name='MinerList.public_keys', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=84,
  serialized_end=116,
)


_HASH = _descriptor.Descriptor(
  name='Hash',
  full_name='Hash',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='Value', full_name='Hash.Value', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=118,
  serialized_end=139,
)


_ADDRESS = _descriptor.Descriptor(
  name='Address',
  full_name='Address',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='Value', full_name='Address.Value', index=0,
      number=1, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=141,
  serialized_end=165,
)


_TRANSACTION = _descriptor.Descriptor(
  name='Transaction',
  full_name='Transaction',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='From', full_name='Transaction.From', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='To', full_name='Transaction.To', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='RefBlockNumber', full_name='Transaction.RefBlockNumber', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='RefBlockPrefix', full_name='Transaction.RefBlockPrefix', index=3,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='MethodName', full_name='Transaction.MethodName', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Params', full_name='Transaction.Params', index=5,
      number=6, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='Signature', full_name='Transaction.Signature', index=6,
      number=10000, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=168,
  serialized_end=331,
)


_ELECTIONVOTINGRECORD = _descriptor.Descriptor(
  name='ElectionVotingRecord',
  full_name='ElectionVotingRecord',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='voter', full_name='ElectionVotingRecord.voter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='candidate', full_name='ElectionVotingRecord.candidate', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='amount', full_name='ElectionVotingRecord.amount', index=2,
      number=3, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='term_number', full_name='ElectionVotingRecord.term_number', index=3,
      number=4, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vote_id', full_name='ElectionVotingRecord.vote_id', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='lock_time', full_name='ElectionVotingRecord.lock_time', index=5,
      number=7, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='unlock_timestamp', full_name='ElectionVotingRecord.unlock_timestamp', index=6,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='withdraw_timestamp', full_name='ElectionVotingRecord.withdraw_timestamp', index=7,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='vote_timestamp', full_name='ElectionVotingRecord.vote_timestamp', index=8,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='is_withdrawn', full_name='ElectionVotingRecord.is_withdrawn', index=9,
      number=13, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=334,
  serialized_end=664,
)


_ELECTORVOTE = _descriptor.Descriptor(
  name='ElectorVote',
  full_name='ElectorVote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='active_voting_record_ids', full_name='ElectorVote.active_voting_record_ids', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='withdrawn_voting_record_ids', full_name='ElectorVote.withdrawn_voting_record_ids', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='active_voted_votes_amount', full_name='ElectorVote.active_voted_votes_amount', index=2,
      number=3, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='all_voted_votes_amount', full_name='ElectorVote.all_voted_votes_amount', index=3,
      number=4, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='active_voting_records', full_name='ElectorVote.active_voting_records', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='withdrawn_votes_records', full_name='ElectorVote.withdrawn_votes_records', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key', full_name='ElectorVote.public_key', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=667,
  serialized_end=962,
)


_CANDIDATEVOTE = _descriptor.Descriptor(
  name='CandidateVote',
  full_name='CandidateVote',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='obtained_active_voting_record_ids', full_name='CandidateVote.obtained_active_voting_record_ids', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='obtained_withdrawn_voting_record_ids', full_name='CandidateVote.obtained_withdrawn_voting_record_ids', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='obtained_active_voted_votes_amount', full_name='CandidateVote.obtained_active_voted_votes_amount', index=2,
      number=3, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='all_obtained_voted_votes_amount', full_name='CandidateVote.all_obtained_voted_votes_amount', index=3,
      number=4, type=18, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='obtained_active_voting_records', full_name='CandidateVote.obtained_active_voting_records', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='obtained_withdrawn_votes_records', full_name='CandidateVote.obtained_withdrawn_votes_records', index=5,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='public_key', full_name='CandidateVote.public_key', index=6,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value=_b(""),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=965,
  serialized_end=1316,
)


_PUBLICKEYSLIST = _descriptor.Descriptor(
  name='PublicKeysList',
  full_name='PublicKeysList',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='value', full_name='PublicKeysList.value', index=0,
      number=1, type=12, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1318,
  serialized_end=1349,
)

_TRANSACTION.fields_by_name['From'].message_type = _ADDRESS
_TRANSACTION.fields_by_name['To'].message_type = _ADDRESS
_ELECTIONVOTINGRECORD.fields_by_name['voter'].message_type = _ADDRESS
_ELECTIONVOTINGRECORD.fields_by_name['vote_id'].message_type = _HASH
_ELECTIONVOTINGRECORD.fields_by_name['unlock_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ELECTIONVOTINGRECORD.fields_by_name['withdraw_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ELECTIONVOTINGRECORD.fields_by_name['vote_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_ELECTORVOTE.fields_by_name['active_voting_record_ids'].message_type = _HASH
_ELECTORVOTE.fields_by_name['withdrawn_voting_record_ids'].message_type = _HASH
_ELECTORVOTE.fields_by_name['active_voting_records'].message_type = _ELECTIONVOTINGRECORD
_ELECTORVOTE.fields_by_name['withdrawn_votes_records'].message_type = _ELECTIONVOTINGRECORD
_CANDIDATEVOTE.fields_by_name['obtained_active_voting_record_ids'].message_type = _HASH
_CANDIDATEVOTE.fields_by_name['obtained_withdrawn_voting_record_ids'].message_type = _HASH
_CANDIDATEVOTE.fields_by_name['obtained_active_voting_records'].message_type = _ELECTIONVOTINGRECORD
_CANDIDATEVOTE.fields_by_name['obtained_withdrawn_votes_records'].message_type = _ELECTIONVOTINGRECORD
DESCRIPTOR.message_types_by_name['StringInput'] = _STRINGINPUT
DESCRIPTOR.message_types_by_name['MinerList'] = _MINERLIST
DESCRIPTOR.message_types_by_name['Hash'] = _HASH
DESCRIPTOR.message_types_by_name['Address'] = _ADDRESS
DESCRIPTOR.message_types_by_name['Transaction'] = _TRANSACTION
DESCRIPTOR.message_types_by_name['ElectionVotingRecord'] = _ELECTIONVOTINGRECORD
DESCRIPTOR.message_types_by_name['ElectorVote'] = _ELECTORVOTE
DESCRIPTOR.message_types_by_name['CandidateVote'] = _CANDIDATEVOTE
DESCRIPTOR.message_types_by_name['PublicKeysList'] = _PUBLICKEYSLIST
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StringInput = _reflection.GeneratedProtocolMessageType('StringInput', (_message.Message,), dict(
  DESCRIPTOR = _STRINGINPUT,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:StringInput)
  ))
_sym_db.RegisterMessage(StringInput)

MinerList = _reflection.GeneratedProtocolMessageType('MinerList', (_message.Message,), dict(
  DESCRIPTOR = _MINERLIST,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:MinerList)
  ))
_sym_db.RegisterMessage(MinerList)

Hash = _reflection.GeneratedProtocolMessageType('Hash', (_message.Message,), dict(
  DESCRIPTOR = _HASH,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:Hash)
  ))
_sym_db.RegisterMessage(Hash)

Address = _reflection.GeneratedProtocolMessageType('Address', (_message.Message,), dict(
  DESCRIPTOR = _ADDRESS,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:Address)
  ))
_sym_db.RegisterMessage(Address)

Transaction = _reflection.GeneratedProtocolMessageType('Transaction', (_message.Message,), dict(
  DESCRIPTOR = _TRANSACTION,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:Transaction)
  ))
_sym_db.RegisterMessage(Transaction)

ElectionVotingRecord = _reflection.GeneratedProtocolMessageType('ElectionVotingRecord', (_message.Message,), dict(
  DESCRIPTOR = _ELECTIONVOTINGRECORD,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:ElectionVotingRecord)
  ))
_sym_db.RegisterMessage(ElectionVotingRecord)

ElectorVote = _reflection.GeneratedProtocolMessageType('ElectorVote', (_message.Message,), dict(
  DESCRIPTOR = _ELECTORVOTE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:ElectorVote)
  ))
_sym_db.RegisterMessage(ElectorVote)

CandidateVote = _reflection.GeneratedProtocolMessageType('CandidateVote', (_message.Message,), dict(
  DESCRIPTOR = _CANDIDATEVOTE,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:CandidateVote)
  ))
_sym_db.RegisterMessage(CandidateVote)

PublicKeysList = _reflection.GeneratedProtocolMessageType('PublicKeysList', (_message.Message,), dict(
  DESCRIPTOR = _PUBLICKEYSLIST,
  __module__ = 'types_pb2'
  # @@protoc_insertion_point(class_scope:PublicKeysList)
  ))
_sym_db.RegisterMessage(PublicKeysList)


# @@protoc_insertion_point(module_scope)



import hashlib

import base58
import requests
from coincurve import PrivateKey

from aelf.types_pb2 import Transaction, Hash, Address, MinerList, StringInput, CandidateVote, PublicKeysList


class AElf(object):
    _get_request_header = None
    _post_request_header = None

    _url = None
    _private_key = None
    _version = None

    def __init__(self, url='http://127.0.0.1:8000', private_key=None, version=None):
        self._url = '%s/api' % url
        if private_key is not None:
            self._private_key = PrivateKey(bytes(bytearray.fromhex(private_key)))

        version = '' if version is None else ';v=%s' % version
        self._post_request_header = {'Content-Type': 'application/json' + version}
        self._get_request_header = {'Accept': 'application/json' + version}

    def get_chain_status(self):
        """
        Get chain status
        :return: chain status
        """
        return requests.get('%s/blockchain/chainStatus' % self._url, headers=self._get_request_header).json()

    def get_block_height(self):
        """
        Get block height
        :return: block height
        """
        block_height = requests.get('%s/blockchain/blockHeight' % self._url, headers=self._get_request_header).content
        return int(block_height)

    def get_block(self, block_hash, include_transactions=False):
        """
        Get block
        :param block_hash: block hash
        :param include_transactions: whether include transactions
        :return: block
        """
        api = '%s/blockchain/block?includeTransactions=%s&blockHash=%s' % (self._url, include_transactions, block_hash)
        return requests.get(api, headers=self._get_request_header).json()

    def get_block_by_height(self, block_height, include_transactions=False):
        """
        Get block by height
        :param block_height: block height
        :param include_transactions: whether include transaction
        :return: block
        """
        api = '%s/blockchain/blockByHeight?includeTransactions=%s&blockHeight=%s' % (
            self._url, include_transactions, block_height)
        return requests.get(api, headers=self._get_request_header).json()

    def get_transaction_pool_status(self):
        """
        Get transaction pool status
        :return: transaction pool status
        """
        return requests.get('%s/blockchain/transactionPoolStatus' % self._url, headers=self._get_request_header).json()

    def create_raw_transaction(self, transaction):
        """
        Create raw transaction
        :param transaction: the json format transaction
            {
              "From": "string",
              "To": "string",
              "RefBlockNumber": 0,
              "RefBlockHash": "string",
              "MethodName": "string",
              "Params": "string"
            }
        :return: the raw transaction string
        """
        return requests.post('%s/blockchain/rawTransaction' % self._url,
                             json=transaction, headers=self._post_request_header).json()

    def send_transaction(self, transaction):
        """
        Send transaction
        :param transaction: transaction hex string
        :return: transaction id
        """
        return requests.post('%s/blockchain/sendTransaction' % self._url,
                             json={'RawTransaction': transaction}, headers=self._post_request_header).json()

    def send_raw_transaction(self, raw_transaction):
        """
        Send raw transaction
        :param raw_transaction: the json format transaction
            {
              "Transaction": "string",
              "Signature": "string",
              "ReturnTransaction": true
            }
        :return: transaction id
        """
        return requests.post('%s/blockchain/sendRawTransaction' % self._url,
                             json=raw_transaction, headers=self._post_request_header).json()

    def send_transactions(self, transactions):
        """
        Send transactions
        :param transactions: transactions (join by ',')
        :return: the list of transaction ids
        """
        return requests.post('%s/blockchain/sendTransaction' % self._url,
                             json={'RawTransactions': transactions}, headers=self._post_request_header).json()

    def execute_transaction(self, transaction):
        """
        Execute transaction
        :param transaction: Transaction object or transaction hex string
        :return: executed result
        """
        if isinstance(transaction, Transaction):
            transaction = transaction.SerializePartialToString().hex()

        response = requests.post('%s/blockchain/executeTransaction' % self._url,
                                 json={'RawTransaction': transaction},
                                 headers=self._post_request_header)
        return response.content

    def execute_raw_transaction(self, raw_transaction):
        """
        Execute raw transaction
        :param raw_transaction: raw transaction
        :return: executed result
        """
        return requests.post('%s/blockchain/executeRawTransaction' % self._url,
                             json=raw_transaction, headers=self._post_request_header).json()

    def get_transaction_result(self, transaction_id):
        """
        Get transaction result
        :param transaction_id: transaction id
        :return: transaction result
        """
        api = '%s/blockchain/transactionResult?transactionId=%s' % (self._url, transaction_id)
        return requests.get(api, headers=self._get_request_header).json()

    def get_transaction_results(self, block_hash):
        """
        Get transaction results
        :param block_hash: block hash
        :return: transaction results
        """
        api = '%s/blockchain/transactionResults?blockHash=%s' % (self._url, block_hash)
        return requests.get(api, headers=self._get_request_header).json()

    def get_peers(self):
        """
        Get peers
        """
        return requests.get('%s/net/peers' % self._url, headers=self._get_request_header).json()

    def add_peer(self, peer_address):
        """
        Add peer
        :param peer_address: peer address
        :return: True/False
        """
        json_data = {'Address': peer_address}
        return requests.post('%s/net/peer' % self._url, json=json_data, headers=self._post_request_header).json()

    def remove_peer(self, address):
        """
        Remove peer
        :param address: peer address
        :return: True/False
        """
        api = '%s/net/peer?address=%s' % (self._url, address)
        status_code = requests.delete(api, headers=self._get_request_header).status_code
        return status_code == 200

    def get_network_info(self):
        """
        Get network info
        :return: network info
        """
        return requests.get('%s/net/networkInfo' % self._url, headers=self._get_request_header).json()

    def get_task_queue_status(self):
        """
        Get task queue status
        :return: task queue status
        """
        return requests.get('%s/blockchain/taskQueueStatus' % self._url, headers=self._get_request_header).json()

    def get_merkle_path(self, transaction_id):
        """
        Get task queue status
        :return: task queue status
        """
        api = '%s/blockChain/merklePathByTransactionId?transactionId=%s' % (self._url, transaction_id)
        return requests.get(api, headers=self._get_request_header).json()

    def get_current_miners(self):
        """
        Get current miners
        :return: current miners
        """
        transaction = self._get_miners_transaction()
        raw_miner_list = self.execute_transaction(transaction)

        current_miners = []
        miner_list = MinerList()
        miner_list.ParseFromString(bytes.fromhex(raw_miner_list.decode()))
        for miner in miner_list.public_keys:
            address = self._get_from_address(miner)
            current_miners.append({
                'public_key': miner.hex(),
                'address': base58.b58encode_check(address.Value)
            })
        return current_miners

    def get_from_address(self):
        """
        Get self from address
        :return: from address (Address object)
        """
        return self._get_from_address(self._private_key.public_key.format(compressed=False))

    def get_from_address_string(self):
        """
        Get self from address string
        :return: from address string
        """
        address = self.get_from_address()
        return base58.b58encode_check(address.Value).decode()

    def get_system_contract_address(self, contract_name):
        """
        Get system contract address
        :param contract_name: system contract name
        :return: contract address object
        """
        transaction = self._get_contract_address_transaction(contract_name)
        raw_address_hex = self.execute_transaction(transaction)
        to_address = Address()
        to_address.ParseFromString(bytes.fromhex(raw_address_hex.decode()))
        return to_address

    def get_system_contract_address_string(self, contract_name):
        """
        Get system contract address
        :param contract_name: system contract name
        :return: contract address base58 string
        """
        to_address = self.get_system_contract_address(contract_name)
        return base58.b58encode_check(to_address.Value).decode()

    def get_candidates(self):
        """
        Get candidates
        :return: candidates
        """
        candidates = []
        transaction = self._get_candidates_transaction()
        raw_candidates = self.execute_transaction(transaction)
        public_key_list = PublicKeysList()
        public_key_list.ParseFromString(bytes.fromhex(raw_candidates.decode()))
        for public_key in public_key_list.value:
            address = self._get_from_address(public_key)
            candidates.append({
                'public_key': public_key.hex(),
                'address': base58.b58encode_check(address.Value)
            })
        return candidates

    def get_vote_info(self, public_keys):
        """
        Get vote info
        :param public_keys: public key for candidates/miners
        :return:
        """
        vote_info = []
        for public_key in public_keys:
            transaction = self._get_candidate_vote_transaction(public_key)
            raw_candidate_vote = self.execute_transaction(transaction)
            candidate_vote = CandidateVote()
            candidate_vote.ParseFromString(bytes.fromhex(raw_candidate_vote.decode()))
            vote_info.append({
                'obtained_active_voted_votes_amount': candidate_vote.obtained_active_voted_votes_amount,
                'all_obtained_voted_votes_amount': candidate_vote.all_obtained_voted_votes_amount
            })
        return vote_info

    def build_transaction(self, to_address, method_name, params=None):
        """
        Build transaction
        :param to_address: to address
        :param method_name: method name
        :param params: params for method
        :return: transaction object
        """
        assert self._private_key is not None, 'To execute transaction, please initialize AElf with private key.'

        chain_status = self.get_chain_status()
        best_chain_hash = chain_status['BestChainHash']
        best_chain_height = chain_status['BestChainHeight']

        transaction = Transaction()
        transaction.From.CopyFrom(self._get_from_address(self._private_key.public_key.format(compressed=False)))
        transaction.To.CopyFrom(to_address)
        transaction.MethodName = method_name
        if params is not None:
            transaction.Params = params
        transaction.RefBlockNumber = best_chain_height
        transaction.RefBlockPrefix = bytes(bytearray.fromhex(best_chain_hash))
        transaction.Signature = self.sign(transaction.SerializeToString())
        return transaction

    def sign(self, bytes_data):
        """
        Sign
        :param bytes_data: the data need sign
        :return: the signed data
        """
        return self._private_key.sign_recoverable(bytes_data)

    def _get_genesis_contract_address(self):
        """ get genesis contract address
        """
        response = requests.get('%s/blockchain/chainStatus' % self._url, headers=self._get_request_header)
        chain_status = response.json()
        genesis_contract_address = chain_status['GenesisContractAddress']
        return genesis_contract_address

    def _get_from_address(self, public_key):
        """ get from address
        """
        address = Address()
        public_key_hash = hashlib.sha256()
        public_key_hash.update(hashlib.sha256(public_key).digest())
        address.Value = public_key_hash.digest()
        return address

    def _get_contract_address_transaction(self, contract_name):
        """ build get contract address transaction
        """
        to_address_bytes = base58.b58decode_check(self._get_genesis_contract_address())
        to_address = Address()
        to_address.Value = to_address_bytes
        params = Hash()
        params.Value = hashlib.sha256(contract_name.encode('utf8')).digest()
        return self.build_transaction(to_address, 'GetContractAddressByName', params.SerializeToString())

    def _get_candidates_transaction(self):
        """ get candidates transaction
        """
        transaction = self._get_contract_address_transaction('AElf.ContractNames.Election')
        address = self.execute_transaction(transaction)
        to_address = Address()
        to_address.ParseFromString(bytes.fromhex(address.decode()))
        return self.build_transaction(to_address, 'GetCandidates')

    def _get_candidate_vote_transaction(self, block_producer):
        """ get candidate vote transaction
        """
        transaction = self._get_contract_address_transaction('AElf.ContractNames.Election')
        address = self.execute_transaction(transaction)
        to_address = Address()
        to_address.ParseFromString(bytes.fromhex(address.decode()))
        params = StringInput()
        params.StringValue = block_producer
        return self.build_transaction(to_address, 'GetCandidateVote', params.SerializeToString())

    def _get_miners_transaction(self):
        """ build get miners transaction
        """
        transaction = self._get_contract_address_transaction('AElf.ContractNames.Consensus')
        raw_address_hex = self.execute_transaction(transaction)
        to_address = Address()
        to_address.ParseFromString(bytes.fromhex(raw_address_hex.decode()))
        return self.build_transaction(to_address, 'GetCurrentMinerList')
